#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['brioche']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['pandas ~= 1.1', 'gspread ~= 3.6', 'numpy ~= 1.1']

entry_points = \
{'console_scripts': ['brioche = brioche.tool:main']}

setup(name='brioche',
      version='0.2.1',
      description='Python/Pandas library to do biomization analyses of pollen samples',
      author='Peter Liljenberg',
      author_email='peter.liljenberg@gmail.com',
      url='https://github.com/petli/brioche',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='~=3.6',
     )
